
/******************************************************************************
* MODULE     : input.gen.cc
* DESCRIPTION: Generic TeXmacs input
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <path.gen.h>
#include <convert.gen.h>

#module code_input
#import path
#import convert

#define DATA_BEGIN   ((char) 2)
#define DATA_END     ((char) 5)
#define DATA_ESCAPE  ((char) 27)

#define STATUS_NORMAL 0
#define STATUS_ESCAPE 1
#define STATUS_BEGIN  2

#define MODE_VERBATIM 0
#define MODE_LATEX    1
#define MODE_HTML     2

/******************************************************************************
* Universal data input
******************************************************************************/

struct texmacs_input_rep: concrete_struct {
  bool   status;
  path   mode;
  string buf;
  tree   doc;
  bool   generic_flag;

  texmacs_input_rep (string the_mode);
  int  get_mode (string s);
  bool put (char c);
  void eof ();
  void flush (bool force= FALSE);
  void write (tree t);
  tree get ();

  void verbatim_flush (bool force= FALSE);
  void latex_flush (bool force= FALSE);
  void html_flush (bool force= FALSE);
};

texmacs_input_rep::texmacs_input_rep (string the_mode):
  status (STATUS_NORMAL), mode (get_mode (the_mode)),
  buf (""), doc (tree (DOCUMENT, "")),
  generic_flag (the_mode == "generic") {}

class texmacs_input {
#import concrete (texmacs_input, texmacs_input_rep)
  texmacs_input (string the_mode);
};
#import code_concrete (texmacs_input, texmacs_input_rep)

texmacs_input::texmacs_input (string the_mode):
  rep (new texmacs_input_rep (the_mode)) {}

/******************************************************************************
* Main routines
******************************************************************************/

int
texmacs_input_rep::get_mode (string s) {
  if (s == "latex") return MODE_LATEX;
  if (s == "html")  return MODE_HTML;
  return MODE_VERBATIM;
}

bool
texmacs_input_rep::put (char c) { // returns TRUE when expecting input
  if (generic_flag) {
    bool block_done= FALSE;
    switch (status) {
    case STATUS_NORMAL:
      if (c == DATA_ESCAPE) status= STATUS_ESCAPE;
      else if (c == DATA_BEGIN) {
	flush (TRUE);
	status= STATUS_BEGIN;
      }
      else if (c == DATA_END) {
	flush (TRUE);
	if (!atom (mode)) mode= mode->next;
	block_done= atom (mode);
      }
      else buf << c;
      break;
    case STATUS_ESCAPE:
    buf << c;
    status= STATUS_NORMAL;
    break;
    case STATUS_BEGIN:
      if (c == ':') {
	mode  = path (get_mode (buf), mode);
	buf   = "";
	status= STATUS_NORMAL;
      }
    else buf << c;
      break;
    }
    if (status == STATUS_NORMAL) flush ();
    return block_done;
  }
  else {
    flush ();
    if (c == DATA_END) return TRUE;
    buf << c;
  }
  return FALSE;
}

void
texmacs_input_rep::eof () {
  flush (TRUE);
}

void
texmacs_input_rep::flush (bool force) {
  switch (mode->item) {
  case MODE_LATEX:
    latex_flush (force);
    break;
  case MODE_HTML:
    html_flush (force);
    break;
  default:
    verbatim_flush (force);
    break;
  }
}

void
texmacs_input_rep::write (tree u) {
  tree& t (doc);
  if (!is_document (u)) u= tree (DOCUMENT, u);
  if (t[N(t)-1] == "") t[N(t)-1]= u[0];
  else {
    if (!is_concat (t[N(t)-1])) t[N(t)-1]= tree (CONCAT, t[N(t)-1]);
    if (!is_concat (u[0])) u[0]= tree (CONCAT, u[0]);
    t[N(t)-1] << u[0]->a;
  }
  t << u(1,N(u))->a;
}

tree
texmacs_input_rep::get () {
  if (doc == tree (DOCUMENT, "")) return "";
  tree t= doc;
  doc= tree (DOCUMENT, "");
  return t;
}

/******************************************************************************
* Flushing
******************************************************************************/

void
texmacs_input_rep::verbatim_flush (bool force) {
  if (force || ends (buf, "\n")) {
    write (verbatim_to_edit (buf));
    buf= "";
  }
}

void
texmacs_input_rep::latex_flush (bool force) {
  if (force || ends (buf, "\n\n")) {
    write (latex_to_edit (buf, "text"));
    buf= "";
  }
}

void
texmacs_input_rep::html_flush (bool force) {
  if (force || ends (buf, "</P>")) {
    write (html_to_edit (buf, "text"));
    buf= "";
  }
}

/******************************************************************************
* User interface
******************************************************************************/

tree
get_texmacs_input (string in, string format) {
  int i;
  texmacs_input tm_in (format);
  for (i=0; i<N(in); i++)
    (void) tm_in->put (in[i]);
  tm_in->eof ();
  return tm_in->get ();
}

#endmodule // code_input
