
/******************************************************************************
* MODULE     : text.gen.cc
* DESCRIPTION: text boxes
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <Boxes/boxes.gen.h>
#include <font.gen.h>

#module code_text_boxes
#import boxes
#import font

/******************************************************************************
* Text boxes
******************************************************************************/

struct text_box_rep: public box_rep {
  string    str;
  font      fn;
  color     col;

  text_box_rep (string s, font fn, color col);
  operator tree () { return str; }

  void      display (ps_device dev);
  path      get_path (SI x, SI y, SI delta);
  cursor    get_cursor (path p);
  path      right_path ();
  selection select (path start, path end);
  double    left_slope ();
  double    right_slope ();
  SI        sub_base ();
  SI        sup_base ();
  SI        right_correction ();
  SI        right_sub_correction (double level);
  SI        right_sup_correction (double level);
};

/******************************************************************************
* Routines for text boxes
******************************************************************************/

text_box_rep::text_box_rep (string s, font fn2, color col2):
  str (s), fn (fn2), col (col2)
{
  text_extents ex;
  fn->get_extents (str, ex);
  x1= ex->x1; y1= ex->y1;
  x2= ex->x2; y2= ex->y2;
  x3= ex->x3; y3= ex->y3;
  x4= ex->x4; y4= ex->y4;
}

path
text_box_rep::right_path () {
  return path (N(str));
}

void
text_box_rep::display (ps_device dev) {
  dev->set_color (col);
  fn->draw (dev, str, 0, 0);
}

path
text_box_rep::get_path (SI x, SI y, SI delta) { (void) y;
  int prev_i, prev_x=0, i=0;
  text_extents ex;
  while (i<N(str)) {
    prev_i= i;
    if (str[i]=='<')
      while ((i<N(str)) && (str[i]!='>'))
	i++;
    i++;

    fn->get_extents (str (0, i), ex);
    int m= (prev_x+ex->x2)>>1;
    if ((x<m) || ((x==m) && (delta<0))) return prev_i;
    prev_x=ex->x2;
  }
  return path (i);
}

cursor
text_box_rep::get_cursor (path p) {
  text_extents ex;
  cursor cu (0, 0);
  fn->get_extents (str (0, p->item), ex);
  double slope= fn->get_right_slope (str (0, p->item));
  cu->x0= ex->x2;
  cu->x1= (SI) (ex->y1* slope);
  cu->y1= ex->y1;
  cu->x2= (SI) (ex->y2* slope);
  cu->y2= ex->y2;
  return cu;
}

selection
text_box_rep::select (path start, path end) {
  SI x1, y1, x2, y2;
  text_extents ex;
  fn->get_extents (str (0, start->item), ex);
  x1= ex->x2;
  y1= ex->y1;
  fn->get_extents (str (0, end->item), ex);
  x2= ex->x2;
  y2= ex->y2;
  return selection (rectangle (x0+x1, y0+y1, x0+x2, y0+y2), start, end);
}

double text_box_rep::left_slope () {
  return fn->get_left_slope (str); }
double text_box_rep::right_slope () {
  return fn->get_right_slope (str); }
SI text_box_rep::sub_base () {
  return fn->get_sub_base (str); }
SI text_box_rep::sup_base () {
  return fn->get_sup_base (str); }
SI text_box_rep::right_sub_correction (double level) { (void) level;
  return - box_rep::left_correction (); }
SI text_box_rep::right_sup_correction (double level) { (void) level;
  return right_correction ()- box_rep::right_correction (); }
SI text_box_rep::right_correction () {
  return fn->get_right_correction (str); }

/******************************************************************************
* Exported routines
******************************************************************************/

static string
get_delimiter (string s, font fn, SI height, bool sqrt_flag) {
  if ((N(s)<2) || (s[0]!='<') || (s[N(s)-1]!='>'))
    fatal_error ("Invalid rubber character", "make_large_delimiter");
  string radical= s (0, N(s)-1) * "-";
  string first  = radical * "0>";
  text_extents ex;
  fn->get_extents (first, ex);
  if ((ex->y2- ex->y1) >= height) return first;

  string second  = radical * "1>";
  fn->get_extents (second, ex);
  int h1= ex->y2- ex->y1;
  if (h1 >= height) return second;

  string third = radical * "2>";
  text_extents ey;
  fn->get_extents (third, ey);
  int h2= ey->y2- ey->y1;
  if (h2 <= h1) return second;
  int d = h2- h1;
  SI n= sqrt_flag?
          ((height + (d-1) - h1) / d):
          ((height + (d>>1) - h1) / d);
  return radical * as_string (n+1) * ">";
}

box
delimiter_box (string s, font fn, color col, SI bot, SI top, bool sqrt_flag) {
  string r= get_delimiter (s, fn, top-bot, sqrt_flag);
  text_extents ex;
  fn->get_extents (r, ex);
  SI y= (top+ bot- ex->y1- ex->y2) >> 1;
  return symbol_box (move_box (text_box (r, fn, col), 0, y), N(s));
}

box
big_operator_box (string s, font fn, color col, int n) {
  if ((N(s)<2) || (s[0]!='<') || (s[N(s)-1]!='>'))
    fatal_error ("Invalid rubber character", "make_large_delimiter");
  string r= s (0, N(s)-1) * "-" * as_string (n) * ">";
  return symbol_box (text_box (r, fn, col), N(s));
}

box
text_box (string s, font fn, color col) {
  return new text_box_rep (s, fn, col);
}

#endmodule // code_text_boxes
